@@ -16,6 +16,7 @@
  * License along with this library. If not, see <http://www.gnu.org/licenses/>.
  */
 
+#include "config.h"
 #import "GdkQuartzNSWindow.h"
 #include "gdkquartzwindow.h"
 #include "gdkdnd-quartz.h"
@@ -74,6 +75,7 @@
 {
   GdkWindow *window = [[self contentView] gdkWindow];
 
+  gdk_synthesize_window_state (window, 0, GDK_WINDOW_STATE_FOCUSED);
   _gdk_quartz_events_update_focus_window (window, TRUE);
 }
 
@@ -82,6 +84,7 @@
   GdkWindow *window = [[self contentView] gdkWindow];
 
   _gdk_quartz_events_update_focus_window (window, FALSE);
+  gdk_synthesize_window_state (window, GDK_WINDOW_STATE_FOCUSED, 0);
 }
 
 -(void)windowDidBecomeMain:(NSNotification *)aNotification
@@ -117,6 +120,13 @@
   inMove = YES;
 }
 
+
+#ifdef AVAILABLE_MAC_OS_X_VERSION_10_12_AND_LATER
+#define NSLeftMouseUp NSEventTypeLeftMouseUp
+#define NSLeftMouseDragged NSEventTypeLeftMouseDragged
+#define NSMouseEntered NSEventTypeMouseEntered
+#endif
+
 -(void)sendEvent:(NSEvent *)event
 {
   switch ([event type])
@@ -186,7 +196,6 @@
   GdkWindow *window = [[self contentView] gdkWindow];
   GdkEvent *event;
 
-  GdkWindowImplQuartz *impl = GDK_WINDOW_IMPL_QUARTZ (window->impl);
   gboolean maximized = gdk_window_get_state (window) & GDK_WINDOW_STATE_MAXIMIZED;
 
   /* In case the window is changed when maximized remove the maximized state */
@@ -217,7 +226,6 @@
   NSRect content_rect = [self contentRectForFrameRect:[self frame]];
   GdkWindow *window = [[self contentView] gdkWindow];
   GdkEvent *event;
-  GdkWindowImplQuartz *impl = GDK_WINDOW_IMPL_QUARTZ (window->impl);
   gboolean maximized = gdk_window_get_state (window) & GDK_WINDOW_STATE_MAXIMIZED;
 
   /* see same in windowDidMove */
@@ -253,7 +261,15 @@
   [self checkSendEnterNotify];
 }
 
--(id)initWithContentRect:(NSRect)contentRect styleMask:(NSUInteger)styleMask backing:(NSBackingStoreType)backingType defer:(BOOL)flag screen:(NSScreen *)screen
+-(id)initWithContentRect:(NSRect)contentRect
+#if MAC_OS_X_VERSION_MIN_REQUIRED < 101200
+               styleMask:(NSUInteger)styleMask
+#else
+               styleMask:(NSWindowStyleMask)styleMask
+#endif
+                 backing:(NSBackingStoreType)backingType
+                   defer:(BOOL)flag
+                  screen:(NSScreen *)screen
 {
   self = [super initWithContentRect:contentRect
 	                  styleMask:styleMask
@@ -363,6 +379,22 @@
 
   initialPositionKnown = NO;
 }
+#if MAC_OS_X_VERSION_MIN_REQUIRED < 101200
+- (NSPoint)convertPointToScreen:(NSPoint)point
+{
+  NSRect inrect = NSMakeRect (point.x, point.y, 0.0, 0.0);
+  NSRect outrect = [self convertRectToScreen: inrect];
+  return (NSPoint)((CGRect)outrect).origin;
+
+}
+
+- (NSPoint)convertPointFromScreen:(NSPoint)point
+{
+  NSRect inrect = NSMakeRect (point.x, point.y, 0.0, 0.0);
+  NSRect outrect = [self convertRectFromScreen: inrect];
+  return (NSPoint)((CGRect)outrect).origin;
+}
+#endif
 
 - (BOOL)trackManualMove
 {
@@ -375,8 +407,11 @@
 
   if (!inManualMove)
     return NO;
-
+#if MAC_OS_X_VERSION_MIN_REQUIRED < 1070
   currentLocation = [self convertBaseToScreen:[self mouseLocationOutsideOfEventStream]];
+#else
+  currentLocation = [self convertPointToScreen:[self mouseLocationOutsideOfEventStream]];
+#endif
   newOrigin.x = currentLocation.x - initialMoveLocation.x;
   newOrigin.y = currentLocation.y - initialMoveLocation.y;
 
@@ -407,7 +442,11 @@
 
   inManualMove = YES;
 
+#if MAC_OS_X_VERSION_MIN_REQUIRED < 1070
   initialMoveLocation = [self convertBaseToScreen:[self mouseLocationOutsideOfEventStream]];
+#else
+  initialMoveLocation = [self convertPointToScreen:[self mouseLocationOutsideOfEventStream]];
+#endif
   initialMoveLocation.x -= frame.origin.x;
   initialMoveLocation.y -= frame.origin.y;
 }
@@ -424,7 +463,11 @@
 
   inTrackManualResize = YES;
 
+#if MAC_OS_X_VERSION_MIN_REQUIRED < 1070
   mouse_location = [self convertBaseToScreen:[self mouseLocationOutsideOfEventStream]];
+#else
+  mouse_location = [self convertPointToScreen:[self mouseLocationOutsideOfEventStream]];
+#endif
   mdx = initialResizeLocation.x - mouse_location.x;
   mdy = initialResizeLocation.y - mouse_location.y;
 
@@ -509,7 +552,12 @@
   resizeEdge = edge;
 
   initialResizeFrame = [self frame];
+
+#if MAC_OS_X_VERSION_MIN_REQUIRED < 1070
   initialResizeLocation = [self convertBaseToScreen:[self mouseLocationOutsideOfEventStream]];
+#else
+  initialResizeLocation = [self convertPointToScreen:[self mouseLocationOutsideOfEventStream]];
+#endif
 }
 
 
@@ -575,7 +623,7 @@ update_context_from_dragging_info (id <NSDraggingInfo> sender)
 
 - (NSDragOperation)draggingEntered:(id <NSDraggingInfo>)sender
 {
-  GdkDeviceManager *device_manager;
+  GdkSeat *seat = NULL;
   GdkEvent *event;
   GdkWindow *window;
 
@@ -589,10 +637,8 @@ update_context_from_dragging_info (id <NSDraggingInfo> sender)
 
   current_context->display = gdk_window_get_display (window);
 
-  device_manager = gdk_display_get_device_manager (gdk_display_get_default ());
-  gdk_drag_context_set_device (current_context,
-                               gdk_device_manager_get_client_pointer (device_manager));
-
+  seat = gdk_display_get_default_seat (gdk_display_get_default ());
+  gdk_drag_context_set_device (current_context, gdk_seat_get_pointer (seat));
   event = gdk_event_new (GDK_DRAG_ENTER);
   event->dnd.window = g_object_ref (window);
   event->dnd.send_event = FALSE;
@@ -644,7 +690,13 @@ update_context_from_dragging_info (id <NSDraggingInfo> sender)
 - (NSDragOperation)draggingUpdated:(id <NSDraggingInfo>)sender
 {
   NSPoint point = [sender draggingLocation];
-  NSPoint screen_point = [self convertBaseToScreen:point];
+  NSPoint screen_point;
+
+#if MAC_OS_X_VERSION_MIN_REQUIRED < 1070
+  screen_point = [self convertBaseToScreen:point];
+#else
+  screen_point = [self convertPointToScreen:point];
+#endif
   GdkEvent *event;
   int gx, gy;
 
@@ -672,7 +724,13 @@ update_context_from_dragging_info (id <NSDraggingInfo> sender)
 - (BOOL)performDragOperation:(id <NSDraggingInfo>)sender
 {
   NSPoint point = [sender draggingLocation];
-  NSPoint screen_point = [self convertBaseToScreen:point];
+  NSPoint screen_point;
+
+#if MAC_OS_X_VERSION_MIN_REQUIRED < 1070
+    screen_point = [self convertBaseToScreen:point];
+#else
+  screen_point = [self convertPointToScreen:point];
+#endif
   GdkEvent *event;
   int gy, gx;
 
@@ -758,17 +816,27 @@ update_context_from_dragging_info (id <NSDraggingInfo> sender)
 }
 
 #ifdef AVAILABLE_MAC_OS_X_VERSION_10_7_AND_LATER
-
+#if MAC_OS_X_VERSION_MIN_REQUIRED < 101200
+typedef enum
+{
+ GDK_QUARTZ_FULL_SCREEN_MASK = NSFullScreenWindowMask,
+} GdkQuartzFullScreen;
+#else
+typedef enum
+{
+ GDK_QUARTZ_FULL_SCREEN_MASK = NSWindowStyleMaskFullScreen,
+} GdkQuartzFullScreen;
+#endif
 - (void)setStyleMask:(NSUInteger)styleMask
 {
   gboolean was_fullscreen;
   gboolean is_fullscreen;
 
-  was_fullscreen = (([self styleMask] & NSFullScreenWindowMask) != 0);
+  was_fullscreen = (([self styleMask] & GDK_QUARTZ_FULL_SCREEN_MASK) != 0);
 
   [super setStyleMask:styleMask];
 
-  is_fullscreen = (([self styleMask] & NSFullScreenWindowMask) != 0);
+  is_fullscreen = (([self styleMask] & GDK_QUARTZ_FULL_SCREEN_MASK) != 0);
 
   if (was_fullscreen != is_fullscreen)
     _gdk_quartz_window_update_fullscreen_state ([[self contentView] gdkWindow]);
@@ -797,7 +865,6 @@ update_context_from_dragging_info (id <NSDraggingInfo> sender)
 {
   NSRect screenFrame = [[self screen] visibleFrame];
   GdkWindow *window = [[self contentView] gdkWindow];
-  GdkWindowImplQuartz *impl = GDK_WINDOW_IMPL_QUARTZ (window->impl);
   gboolean maximized = gdk_window_get_state (window) & GDK_WINDOW_STATE_MAXIMIZED;
 
   if (!maximized)
@@ -811,7 +878,6 @@ update_context_from_dragging_info (id <NSDraggingInfo> sender)
 {
 
   GdkWindow *window = [[self contentView] gdkWindow];
-  GdkWindowImplQuartz *impl = GDK_WINDOW_IMPL_QUARTZ (window->impl);
   gboolean maximized = gdk_window_get_state (window) & GDK_WINDOW_STATE_MAXIMIZED;
 
   if (maximized)
@@ -838,4 +904,19 @@ update_context_from_dragging_info (id <NSDraggingInfo> sender)
   inMaximizeTransition = NO;
 }
 
+-(NSSize)window:(NSWindow *)window willUseFullScreenContentSize:(NSSize)proposedSize
+{
+  return [[window screen] frame].size;
+}
+
+-(void)windowWillEnterFullScreen:(NSNotification *)aNotification
+{
+  lastUnfullscreenFrame = [self frame];
+}
+
+-(void)windowWillExitFullScreen:(NSNotification *)aNotification
+{
+  [self setFrame:lastUnfullscreenFrame display:YES];
+}
+
 @end
