@@ -1042,6 +1042,7 @@ gtk_popover_update_position (GtkPopover *popover)
   GtkPopoverPrivate *priv = popover->priv;
   GtkWidget *widget = GTK_WIDGET (popover);
   GtkAllocation window_alloc;
+  GtkBorder window_shadow;
   GdkRectangle rect;
   GtkRequisition req;
   GtkPositionType pos;
@@ -1054,6 +1055,7 @@ gtk_popover_update_position (GtkPopover *popover)
 
   gtk_widget_get_preferred_size (widget, NULL, &req);
   gtk_widget_get_allocation (GTK_WIDGET (priv->window), &window_alloc);
+  _gtk_window_get_shadow_width (priv->window, &window_shadow);
   priv->final_position = priv->preferred_position;
 
   gtk_popover_get_pointing_to (popover, &rect);
@@ -1062,10 +1064,12 @@ gtk_popover_update_position (GtkPopover *popover)
 
   pos = get_effective_position (popover, priv->preferred_position);
 
-  overshoot[GTK_POS_TOP] = req.height - rect.y;
-  overshoot[GTK_POS_BOTTOM] = rect.y + rect.height + req.height - window_alloc.height;
-  overshoot[GTK_POS_LEFT] = req.width - rect.x;
-  overshoot[GTK_POS_RIGHT] = rect.x + rect.width + req.width - window_alloc.width;
+  overshoot[GTK_POS_TOP] = req.height - rect.y + window_shadow.top;
+  overshoot[GTK_POS_BOTTOM] = rect.y + rect.height + req.height - window_alloc.height
+                              + window_shadow.bottom;
+  overshoot[GTK_POS_LEFT] = req.width - rect.x + window_shadow.left;
+  overshoot[GTK_POS_RIGHT] = rect.x + rect.width + req.width - window_alloc.width
+                             + window_shadow.right;
 
 #ifdef GDK_WINDOWING_WAYLAND
   if (GDK_IS_WAYLAND_DISPLAY (gtk_widget_get_display (widget)) &&
@@ -1757,6 +1761,10 @@ gtk_popover_class_init (GtkPopoverClass *klass)
    * Whether show/hide transitions are enabled for this popover.
    *
    * Since: 3.16
+   *
+   * Deprecated: 3.22: You can show or hide the popover without transitions
+   *   using gtk_widget_show() and gtk_widget_hide() while gtk_popover_popup()
+   *   and gtk_popover_popdown() will use transitions.
    */
   properties[PROP_TRANSITIONS_ENABLED] =
       g_param_spec_boolean ("transitions-enabled",
@@ -1781,6 +1789,14 @@ gtk_popover_class_init (GtkPopoverClass *klass)
 
   g_object_class_install_properties (object_class, NUM_PROPERTIES, properties);
 
+  /**
+   * GtkPopover::closed:
+   *
+   * This signal is emitted when the popover is dismissed either through
+   * API or user interaction.
+   *
+   * Since: 3.12
+   */
   signals[CLOSED] =
     g_signal_new (I_("closed"),
                   G_TYPE_FROM_CLASS (object_class),
